@@ -66,6 +66,10 @@ impl OpendalFs {
     pub fn new(op: Operator) -> Box<OpendalFs> {
         Box::new(OpendalFs { op })
     }
+
+    fn fs_path(&self, path: &DavPath) -> Result<String, FsError> {
+        String::from_utf8(path.as_bytes().to_vec()).map_err(|_| FsError::GeneralFailure)
+    }
 }
 
 impl DavFileSystem for OpendalFs {
@@ -75,7 +79,8 @@ impl DavFileSystem for OpendalFs {
         options: dav_server::fs::OpenOptions,
     ) -> FsFuture<'a, Box<dyn DavFile>> {
         async move {
-            let file = OpendalFile::open(self.op.clone(), path.clone(), options).await?;
+            let path = self.fs_path(path)?;
+            let file = OpendalFile::open(self.op.clone(), path, options).await?;
             Ok(Box::new(file) as Box<dyn DavFile>)
         }
         .boxed()
@@ -87,7 +92,7 @@ impl DavFileSystem for OpendalFs {
         _meta: ReadDirMeta,
     ) -> FsFuture<'a, FsStream<Box<dyn DavDirEntry>>> {
         async move {
-            let path = path.as_url_string();
+            let path = self.fs_path(path)?;
             self.op
                 .lister(path.as_str())
                 .await
@@ -99,7 +104,8 @@ impl DavFileSystem for OpendalFs {
 
     fn metadata<'a>(&'a self, path: &'a DavPath) -> FsFuture<'a, Box<dyn DavMetaData>> {
         async move {
-            let opendal_metadata = self.op.stat(path.as_url_string().as_str()).await;
+            let path = self.fs_path(path)?;
+            let opendal_metadata = self.op.stat(path.as_str()).await;
             match opendal_metadata {
                 Ok(metadata) => {
                     let webdav_metadata = OpendalMetaData::new(metadata);
@@ -113,7 +119,7 @@ impl DavFileSystem for OpendalFs {
 
     fn create_dir<'a>(&'a self, path: &'a DavPath) -> FsFuture<'a, ()> {
         async move {
-            let path = path.as_url_string();
+            let path = self.fs_path(path)?;
 
             // check if the parent path is exist.
             // During MKCOL processing, a server MUST make the Request-URI a member of its parent collection, unless the Request-URI is "/".  If no such ancestor exists, the method MUST fail.
@@ -156,10 +162,8 @@ impl DavFileSystem for OpendalFs {
 
     fn remove_file<'a>(&'a self, path: &'a DavPath) -> FsFuture<'a, ()> {
         async move {
-            self.op
-                .delete(path.as_url_string().as_str())
-                .await
-                .map_err(convert_error)
+            let path = self.fs_path(path)?;
+            self.op.delete(&path).await.map_err(convert_error)
         }
         .boxed()
     }
